"""Collection of one-size-fits-all wisdoms and questionable quotations."""

fortunes = [

    # Technology #

    'The plural of Regex is "Regrets". ',

    'Why make it simple and efficient, '
    'when you can make it a huge overcomplicated mess? ',

    '''"Stable"? That's for horses. '''

    'Roses are red, violets are blue. '
    'Unterminated string literal at line 32. ',

    'A computer is much like air conditioning: '
    'useless the moment you open Windows. ',

    'We built our world without walls and fences, '
    "so naturally we don't need Windows and Gates. ",

    "object: v. to feel strong distaste for; to oppose firmly.\n"
    "See also: object-oriented programming. ",

    "Just a few hours of trial and error can save "
    "countless minutes of reading the documentation. ",

    "Printing is not supported on this printer. ",

    "Required qualifications: minimal Java experience (ideally none)",

    "Configuring login daemon...\n"
    "Starting login daemon...\n"
    "Exorcising login daemon...\n"
    "System operational.",

    "This is a security feature. It secures our revenue stream. ",

    "We need these permissions to enable intergation. "
    "Integration of our backdoor into your OS, that is. ",

    "Our platform is end-to-end encrypted. "
    "The one end is your client, and the other end is our server. ",

    "We have two-factor authentication. "
    "The first factor is your username, and second factor is your password. ",

    "Unix: full of Daemons, ELFs and DWARFs",

    "Step one: solve the problem. Step two: become the problem. "

    # Education #

    '"The doors are wide open, '
    'but we still see people crashing into the doorframe.\n" '
    '    -- NvdM on IC exams',

    'In order to save energy and help meet our goal of a carbon-neutral '
    'campus by 2025, the light at the end of the tunnel has been turned off. ',

    "Technical university: technically, it's a university. ",

    "Theoretical physicists: in theory, they're good at physics. ",

    "Don't try; just do. ",

    "If you can't, then don't. ",

    "Pay attention, I know exactly what I am talking about. ",

    "Just because [pipelining] wasn't covered in the lectures "
    "doesn't meant it won't be on the exam. ",

    "Learning by doing: I do, you learn. ",

    "There are no stupid questions, "
    "only stupid people. ",

    # Politics #

    "Abuse of authority comes as no surprise. ",

    "Money doesn't bring happiness, but neither does poverty. ",

    '"Somehow the potential to make a lot of money has become more valuable '
    'then making a lot of money lol. " -- u/h3J1e on shareholders',

    "Elect a clown, expect a circus. ",

    "Points of discussion: "
    "building a new shed, and building communism.\n"
    "Due to lumber shortages, it was decided to skip "
    "straight to the second point. ",

    'Now is the time to save face. We hope there will be time to '
    'save other parts of the body as well. ',

    "We wanted the best, but it turned out like always. ",

    "No matter what we try to develop, we always end up with either "
    "the communist party or the Khalashnikov rifle.",

    "You can't hug a child with nuclear arms. ",

    'Experimental missile nicknamed "Civil Servant": '
    "won't work and they can't fire it",

    "We have thoroughly addressed all concerns, from A to B. ",

    "He is a man of logic and should not be taken seriously. ",

    "That is my opinion. Do not ask me why. ",

    "- A choice of what? \n"
    "- A choice of watt? \n"
    "- Yes, a choice of what? \n"
    "- No, the watts are exactly the same, "
    "the choice is who you buy them from. ",

    "- How long are you in for?\n"
    "- Three years\n"
    "- What for?\n"
    "- For nothing!\n"
    "- Do not try to play us for fools, "
    'everyone knows that "for nothing" is 5 years. ',

    "Checklist for travel to The Netherlands:\n"
    "    - Do not travel to The Netherlands. ",

    "Freedom of speech is ofcourse guaranteed. "
    "Freedom *after* the speech, however, is a more delicate topic. ",

    'When I commit violence, it is called "crime", '
    'but when you commit violence, it is called "law". '
    "That doesn't seem fair at all! ",

    "Our enemy is weak and pathetic, but also a deadly threat. ",

    "We pretend to work, and they pretend to pay us. ",

    "The advent of communism will be announced "
    "in the newspapers, over the radio, and perhaps even "
    "on television -- provided the people can still "
    "afford television sets, that is.",

    # Malapropisms #

    "Whatever bloats your goat! ",

    # Engineering #

    "Instructions for use: do not. ",

    "Square peg. Round hole. Get it done. ",

    "Emergency phone not installed. "
    "Please do not have an emergency at this location. ",

    'The difference between practice and theory '
    'is often bigger in practice than in theory. ',

    'No keyboard detected. Press F12 to continue.',

    "Forbidden to remove boulder. "
    "Without it, lift does not carry children. ",

    'Anything labeled "this end towards enemy" is dangerous at both ends. ',

    "It works all the time 60% of the time. ",

    "- I'm from quality assurance. "
    "I can assure you we didn't find any in this product.\n"
    "- Any what?\n"
    "- Any quality.\n",

    "Battery life: up to a minimum of 8 hours! ",

    "This seems like a bad idea, but it's fine for now. ",

    "This was a problem for future me, and now I am future me. ",

    # Resilience #

    "Even with everyone's help, you still have to do everything on your own. ",

    "YOU GOTTA BLAME SOMEONE, RIGHT? BLAME YOURSELF",

    "This time it's the same, but also a little bit different. ",

    "It's not a superiority complex if you're actually superior. ",

    "It's not paranoia if someone's actually out to get you. ",

    "The horrors persist, but so do I",

    "New liver, same eagles. ",

    "It is what it is. ",

    "Life is a joke; don't be the punchline. ",

    "Just because you can survive in harsh conditions "
    "doesn't mean you should have to. ",

    "Even if you're a freak of nature, it doesn't mean that you deserve "
    "to be treated like one. ",

    "Tip: spice up your panic attack with a harmonica. ",

    "Don't delay, despair today! ",

    '''Pessimism: "It's only gonna get worse"\n'''
    '''Optimism: "It *can't* get any worse"''',

    "Cheer up! Things are getting worse at a slower rate! ",

    "Everything is going worryingly well. ",

    "It's easy to miss, but once you see it, you cannot unsee it. ",

    "You can't solve problems with the same "
    "type of thinking that created them. ",

    "This is informative and unfortunate. ",

    '''You're not being "represented", you're being marketed to. ''',

    "You have to laugh about these kinds of things, "
    "because otherwise you will cry. ",

    "Why fall in love when you can just fall asleep? ",

    "Why pass your exams when you can just pass away? ",

    # Misc #

    "The best storytellers are those who have a story to tell. ",

    "CAUTION: Gate swings both ways HARD",

    "On one hand, you've got five oranges. On the other hand, you've got "
    "three apples. Conclusion: you have big hands. ",

    "When you are dead, you do not suffer.\n"
    "The pain is only felt by those around you.\n"
    "The same happens when you are stupid. ",

    "He's suffered for his art and now it's our turn. ",

    "i may be cringe but youre mean and thats worse",

    "Qaulifications: former child",

    '"Whenever a door closes, another one surely opens"\n'
    '    -- My uncle (wise man, but a terrible cabinet maker)',

    "Present day, present time. ",

    "As above, so below. ",

    "I am not insulting you, I am *describing* you. ",

    "I hope this email finds you before I do. ",

    "To whom it WILL concern: ",

    "Bipartisan approval of medical cannabis for arthritis patients:\n"
    "joint support for joint support for joint support. ",

    "The bar is so low that it is practically on the ground, "
    "yet here you are, shovels in hand. ",

    "Every triangle is a love triangle if you love triangles\n"
    " -- Pythagoras",

    "On the fifth day, you gave earth its inhabitants. "
    "You made them in your image, which was probably a mistake. ",

    "What part of hunting/gathering is this supposed to be!?",

    "- I'm telling the truth, I swear! You can just read my mind, can't you?\n"
    "- Heh. Only an idiot would think the truth on Kin Dza Dza!",

    "The birds are singing, the sun is shining, my koi fish pond no longer "
    "has algae in it, and I didn't need to use a single chemical, "
    "and thermaltake is screwing over their customers, "
    "and we're gonna make sure they never get a sale again. ",

    'This is an "A Condition" item. '
    "That is to say, it's definitely in a condition. ",

    "I know writers who use subtext, and they're all cowards.",

    "Two-word horror: Medical Entrepreneur",

    ]

